package DrumBoiler "Package containing examples and components used in the example"
  model DrumBoilerModel "Complete drum boiler model, including evaporator and supplementary components"
    extends Modelica.Icons.Example;
    Components.EquilibriumDrumBoiler boiler(m_D = 300e3, cp_D = 500, V_t = 100, V_l_start = 67, redeclare package Medium = Modelica.Media.Water.StandardWater, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 100000) annotation(Placement(transformation(extent = {{-45, -30}, {-25, -10}}, origin = {5, 0}, rotation = 0), visible = true));
    Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow furnace annotation(Placement(transformation(origin = {-31, -51}, extent = {{-9, -11}, {11, 9}}, rotation = 90), visible = true));
    Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, p = Modelica.Units.Conversions.from_bar(0.5), redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, T = 500) annotation(Placement(transformation(origin = {90, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
    Modelica.Fluid.Sensors.MassFlowRate massFlowRate(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(origin = {10, -20}, extent = {{10, 10}, {-10, -10}}, rotation = 180), visible = true));
    Modelica.Fluid.Sources.MassFlowSource_h pump(nPorts = 1, h = 5e5, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true) "Feed water pump" annotation(Placement(transformation(extent = {{-80, -30}, {-60, -10}}, origin = {0, 0}, rotation = 0), visible = true));
    Modelica.Fluid.Valves.ValveLinear steamValve(redeclare package Medium = Modelica.Media.Water.StandardWater, dp_nominal = 9000000, m_flow_nominal = 180) "Valve at the steam outlet" annotation(Placement(transformation(extent = {{50, -10}, {70, -30}}, origin = {-10, 0}, rotation = 0), visible = true));
    inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{-90, 70}, {-70, 90}}, origin = {150, -50}, rotation = 0), visible = true));
    Modelica.Blocks.Sources.Ramp powerInput(height = 400e6, duration = startUpTime) annotation(Placement(visible = true, transformation(origin = {-70, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    parameter Modelica.Units.SI.Time startUpTime(min = 0.0, start = 2) = 2900 "Time taken to reach peak power";
    Modelica.Blocks.Sources.Ramp valveOpening(height = 1, duration = 500) annotation(Placement(visible = true, transformation(origin = {30, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Components.Controller controller annotation(Placement(visible = true, transformation(origin = {-60, 30}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  equation
    connect(furnace.port, boiler.heatPort) annotation(Line(points = {{0, -5}, {0, 5}}, color = {191, 0, 0}, origin = {-30, -35}, visible = true));
    connect(pump.ports[1], boiler.port_a) annotation(Line(points = {{-10, 0}, {10, 0}}, color = {0, 127, 255}, origin = {-50, -20}, visible = true));
    connect(massFlowRate.port_b, steamValve.port_a) annotation(Line(points = {{-10, 0}, {10, 0}}, color = {0, 127, 255}, visible = true, origin = {30, -20}));
    connect(steamValve.port_b, sink.ports[1]) annotation(Line(points = {{60, -20}, {75, -20}, {80, -20}}, color = {0, 127, 255}, visible = true));
    connect(boiler.port_b, massFlowRate.port_a) annotation(Line(points = {{-10, 0}, {10, 0}}, color = {0, 127, 255}, origin = {-10, -20}, visible = true));
    connect(powerInput.y, furnace.Q_flow) annotation(Line(visible = true, origin = {-39.667, -66.667}, points = {{-19.333, -3.333}, {9.667, -3.333}, {9.667, 6.667}}, color = {1, 37, 163}));
    connect(valveOpening.y, steamValve.opening) annotation(Line(visible = true, origin = {47, -56}, points = {{-6, -14}, {3, -14}, {3, 28}}, color = {1, 37, 163}));
    connect(boiler.V, controller.u2) annotation(Line(visible = true, points = {{7.667, -26}, {7.667, 13}, {-15.333, 13}}, color = {1, 37, 163}, origin = {-32.667, 17}));
    connect(pump.m_flow_in, controller.y1) annotation(Line(visible = true, origin = {-85, 9}, points = {{5, -21}, {-15, -21}, {-15, 21}, {14, 21}}, color = {1, 37, 163}));
    annotation(experiment(StartTime = 0, StopTime = 7200, __Wolfram_DisplayTimeUnit = "min"), preferredView = "info", __Wolfram, Documentation(info = "<html>
<p>
The nonlinear drum boiler model is connected to a feedwater pump, a furnace and a valve at the steam outlet. In order to prevent overflow of water inside the boiler, the feedwater needs to be controlled so that the water level is kept at its set point.
</p><p>
The model uses accurate properties of water and steam according to the IAPWS/IF97 standard.
</p>

<html><p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://DrumBoiler/DrumBoiler.nb\">DrumBoiler.nb</a>.</p></html>
</html>", figures = {Figure(title = "Temperature and pressure of steam", identifier = "6de6b", plots = {Plot(curves = {Curve(y = boiler.T, legend = "temperature inside the boiler")}, x = Axis(unit = "s")), Plot(curves = {Curve(y = boiler.p, legend = "pressure inside the boiler")}, x = Axis(unit = "s"))}, caption = "The temperature and pressure at the end of the simulation are %{boiler.T} deg C and %{boiler.p} bar respectively."), Figure(title = "Liquid level in the boiler", identifier = "c89a6", plots = {Plot(curves = {Curve(y = controller.u2, legend = "level of liquid inside boiler"), Curve(y = controller.levelSetPoint.y, legend = "reference level of liquid inside the boiler")})}, caption = "Reference level is set at  %{controller.levelSetPoint.y} m"), Figure(title = "Power supplied to the boiler", identifier = "56141", plots = {Plot(curves = {Curve(y = powerInput.y, legend = "power input in watts")})}, caption = "A ramp signal is used to model the input heat flow from the furnace"), Figure(title = "Thermal stress in the boiler", identifier = "acaf8", plots = {Plot(curves = {Curve(y = boiler.thermal_stress, legend = "Thermal stress")})}, caption = "The magnitude of the thermal stress should be below 10 for safe operation. See see %[Franke, Rode, Kruege](https://www.modelica.org/events/Conference2003/index_html/papers/h29_Franke.pdf) for more information."), Figure(title = "phase volumes", identifier = "d3e33", preferred = true, plots = {Plot(curves = {Curve(y = boiler.V_l, legend = "volume of water")}), Plot(curves = {Curve(y = boiler.V_v, legend = "volume of steam")}), Plot(curves = {Curve(y = furnace.Q_flow, legend = "heat input to boiler")}, y = Axis(unit = "MW"))}, caption = "The top plot shows the volume of water. The middle plot shows the volume of steam. The third plot shows the amount of heat added by the furnace.")}), Diagram(coordinateSystem(extent = {{-110, -95}, {110, 50}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end DrumBoilerModel;

  package Components "Package containing components used by the drum boiler example"
    extends Modelica.Icons.Package;

    model Controller "Model of a PI controller"
      Modelica.Blocks.Continuous.PI PI(T = 120, k = 10, initType = Modelica.Blocks.Types.Init.InitialState) annotation(Placement(transformation(extent = {{-70, 32.857}, {-90, 52.857}}, origin = {20, -12.857}, rotation = 0), visible = true));
      Modelica.Blocks.Math.Feedback feedback annotation(Placement(transformation(extent = {{-22, 20}, {-42, 40}}, origin = {2, 0}, rotation = 0), visible = true));
      Modelica.Blocks.Sources.Constant levelSetPoint(k = 67) annotation(Placement(transformation(extent = {{54.285, 68.572}, {34.285, 88.572}}, origin = {-34.285, -48.572}, rotation = 0), visible = true));
      Modelica.Blocks.Nonlinear.Limiter limiter(uMin = 0, uMax = 500) annotation(Placement(transformation(origin = {-90, 30}, extent = {{-10, 10}, {10, -10}}, rotation = 180), visible = true));
      Modelica.Blocks.Interfaces.RealInput u2 annotation(Placement(visible = true, transformation(origin = {-30, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -270), iconTransformation(origin = {-120, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealOutput y1 annotation(Placement(visible = true, transformation(origin = {-130, 30}, extent = {{-10, -10}, {10, 10}}, rotation = -180), iconTransformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    equation
      connect(PI.u, feedback.y) annotation(Line(points = {{-4.5, 0}, {4.5, 0}}, color = {0, 36, 164}, origin = {-43.5, 30}, visible = true));
      connect(PI.y, limiter.u) annotation(Line(points = {{3.5, 0}, {-3.5, 0}}, color = {1, 37, 163}, visible = true, origin = {-74.5, 30}));
      connect(levelSetPoint.y, feedback.u1) annotation(Line(points = {{10.5, 0}, {-10.5, 0}}, color = {1, 37, 163}, visible = true, origin = {-11.5, 30}));
      connect(u2, feedback.u2) annotation(Line(visible = true, origin = {-30, 1}, points = {{-0, -21}, {0, 21}}, color = {1, 37, 163}));
      connect(limiter.y, y1) annotation(Line(visible = true, origin = {-115.5, 30}, points = {{14.5, 0}, {-14.5, 0}}, color = {1, 37, 163}));
      annotation(Documentation(info = "<html>

    <p>
The model consists of a PI controller along with a set point for water level inside the boiler. It also contains a limiter to limit the range of output from the PI controller.
</p></html>"), Diagram(coordinateSystem(extent = {{-140, -40}, {40, 60}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Text(visible = true, textColor = {64, 64, 64}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Rectangle(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Text(visible = true, origin = {3.504, -10}, textColor = {64, 64, 64}, extent = {{-40, -30}, {40, 50}}, textString = "PI")}));
    end Controller;

    model EquilibriumDrumBoiler "Simple Evaporator with two states, see Astroem, Bell: Drum-boiler dynamics, Automatica 36, 2000, pp.363-378"
      extends Modelica.Fluid.Interfaces.PartialTwoPort(final port_a_exposesState = true, final port_b_exposesState = true, redeclare replaceable package Medium = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialTwoPhaseMedium);
      import Modelica.Constants;
      import Modelica.Fluid.Types;
      parameter Modelica.Units.SI.Mass m_D "mass of surrounding drum metal";
      parameter Medium.SpecificHeatCapacity cp_D "specific heat capacity of drum metal";
      parameter Modelica.Units.SI.Volume V_t "total volume inside drum";
      parameter Medium.AbsolutePressure p_start = system.p_start "Start value of pressure" annotation(Dialog(tab = "Initialization"));
      parameter Modelica.Units.SI.Volume V_l_start = V_t / 2 "Start value of liquid volumeStart value of volume" annotation(Dialog(tab = "Initialization"));
      // Assumptions
      parameter Boolean allowFlowReversal = system.allowFlowReversal "allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
      parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(Placement(transformation(extent = {{-10, -110}, {10, -90}})));
      Modelica.Blocks.Interfaces.RealOutput V(unit = "m3") "liquid volume" annotation(Placement(transformation(origin = {40, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true, iconTransformation(origin = {50, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
      Medium.SaturationProperties sat "State vector to compute saturation properties";
      Medium.AbsolutePressure p(start = p_start, stateSelect = StateSelect.prefer) "pressure inside drum boiler";
      Medium.Temperature T "temperature inside drum boiler";
      Modelica.Units.SI.Volume V_v "volume of vapour phase";
      Modelica.Units.SI.Volume V_l(start = V_l_start, stateSelect = StateSelect.prefer) "volumes of liquid phase";
      Medium.SpecificEnthalpy h_v = Medium.dewEnthalpy(sat) "specific enthalpy of vapour";
      Medium.SpecificEnthalpy h_l = Medium.bubbleEnthalpy(sat) "specific enthalpy of liquid";
      Medium.Density rho_v = Medium.dewDensity(sat) "density in vapour phase";
      Medium.Density rho_l = Medium.bubbleDensity(sat) "density in liquid phase";
      Modelica.Units.SI.Mass m "total mass of drum boiler";
      Modelica.Units.SI.Energy U "internal energy";
      Medium.Temperature T_D = heatPort.T "temperature of drum";
      Modelica.Units.SI.HeatFlowRate q_F = heatPort.Q_flow "heat flow rate from furnace";
      Medium.SpecificEnthalpy h_W = inStream(port_a.h_outflow) "Feed water enthalpy (specific enthalpy close to feedwater port when mass flows in to the boiler)";
      Medium.SpecificEnthalpy h_S = inStream(port_b.h_outflow) "steam enthalpy (specific enthalpy close to steam port when mass flows in to the boiler)";
      Modelica.Units.SI.MassFlowRate qm_W = port_a.m_flow "feed water mass flow rate";
      Modelica.Units.SI.MassFlowRate qm_S = port_b.m_flow "steam mass flow rate";
      /*outer Modelica.Fluid.Components.FluidOptions fluidOptions
    "Global default options";*/
      Real thermal_stress;
    equation
      // Thermal stress
      thermal_stress = der(T) * 60;
      // balance equations
      m = rho_v * V_v + rho_l * V_l + m_D "Total mass";
      U = rho_v * V_v * h_v + rho_l * V_l * h_l - p * V_t + m_D * cp_D * T_D "Total energy";
      if massDynamics == Types.Dynamics.SteadyState then
        0 = qm_W + qm_S "Steady state mass balance";
      else
        der(m) = qm_W + qm_S "Dynamic mass balance";
      end if;
      if energyDynamics == Types.Dynamics.SteadyState then
        0 = q_F + port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow) "Steady state energy balance";
      else
        der(U) = q_F + port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow) "Dynamic energy balance";
      end if;
      V_t = V_l + V_v;
      // Properties of saturated liquid and steam
      sat.psat = p;
      sat.Tsat = T;
      sat.Tsat = Medium.saturationTemperature(p);
      // ideal heat transfer between metal and water
      T_D = T;
      // boundary conditions at the ports
      port_a.p = p;
      port_a.h_outflow = h_l;
      port_b.p = p;
      port_b.h_outflow = h_v;
      // liquid volume
      V = V_l;
      // Check that two-phase equilibrium is actually possible
      assert(p < Medium.fluidConstants[1].criticalPressure - 10000, "Evaporator model requires subcritical pressure");
    initial equation
      // Initial conditions
      // Note: p represents the energy as it is constrained by T_sat
      if energyDynamics == Types.Dynamics.FixedInitial then
        p = p_start;
      elseif energyDynamics == Types.Dynamics.SteadyStateInitial then
        der(p) = 0;
      end if;
      if massDynamics == Types.Dynamics.FixedInitial then
        V_l = V_l_start;
      elseif energyDynamics == Types.Dynamics.SteadyStateInitial then
        der(V_l) = 0;
      end if;
      annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {64, 64, 64}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -64}, {100, 64}}), Rectangle(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -44}, {100, 44}}), Rectangle(visible = true, lineColor = {10, 90, 224}, fillColor = {156, 203, 255}, fillPattern = FillPattern.HorizontalCylinder, extent = DynamicSelect({{-100, -44}, {100, 44}}, {{-100, -44}, {-100 + 200 * V_l / V_t, 44}})), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{18, -29}, {48, 0}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-1, 0}, {29, 29}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{43, 2}, {73, 31}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-31, -28}, {-1, 1}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{50, -14}, {80, 15}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-72, -4}, {-42, 25}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{71, -40}, {101, -11}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{72, -1}, {102, 28}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{71, 11}, {101, 40}}), Line(visible = true, points = {{0, -64}, {0, -100}}, color = {191, 0, 0}), Line(visible = true, origin = {10, 0}, points = {{40, 100}, {40, 64}}, color = {0, 0, 127}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{58, -40}, {88, -11}}), Ellipse(visible = true, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{71, -28}, {101, 1}})}), Documentation(revisions = "<html>
<ul>
<li><em>Dec 2008</em>
    by R&uuml;diger Franke:<br>
     Adapt initialization to new Types.Dynamics</li>
<li><em>2 Nov 2005</em>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
     Initialization options fixed</li>
<li><em>6 Sep 2005</em><br>
    Model by R&uuml;diger Franke<br>
    See Franke, Rode, Kr&uuml;ger: On-line Optimization of Drum Boiler Startup, 3rd International Modelica Conference, Link&ouml;ping, 2003.<br>
    Modified after the 45th Design Meeting</li>
</ul>
</html>", info = "<html>


    <p>
Inside the drum there is water in two phases: liquid and vapour. The thermodynamic state of both phases is assumed to be at the phase boundary. Feed water enters the drum and saturated steam leaves the drum. A furnace supplies energy for heating up and evaporating the feed water model.
</p><p>
The model assumes a global mass balance for feedwater entering and steam leaving the drum. The global energy balance considers energy in stored in the vapor phase, liquid phase, work and thermal energy in the surrounding material.
</p><p>
It is assumed that the specific enthalpy of steam leaving the boiler is equal to the vapour enthalpy. Furthermore, ideal heat transfer between the water inside the drum and surrounding material is assumed. Consequently the metal temperature is equal to the saturation temperature of water for the pressure inside the drum.
</p>

</html>"));
    end EquilibriumDrumBoiler;
    annotation(Documentation, Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0")), __Wolfram(openClass = DrumBoiler.DrumBoilerModel), Documentation(info = "<html><p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://DrumBoiler/DrumBoiler.nb\">DrumBoiler.nb</a>.</p></html>"), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end DrumBoiler;
